require "camping"
require "coderay"
require "fileutils"
require "markaby"
require "mime/types"
require "pathname"

Camping.goes :Plemp

Markaby::Builder.set(:indent, 2)
unless defined? BASE_DIR
  BASE_DIR = Pathname.new(__FILE__).dirname
  PUBLIC_DIR = BASE_DIR + "public"
  UPLOAD_DIR = BASE_DIR + "upload"

  Positions = Hash.new
end

module Plemp::Controllers

  class Index
    def get
      render :main
    end
  end

 class StaticXX
    def get(type, path)
      mime_type = MIME::Types.type_for(path).first 
      @headers['Content-Type'] = mime_type.nil? ? "text/plain" : mime_type.to_s
      unless path.include? ".." or not ["public", "upload"].include? type
        @headers['X-Sendfile'] = (BASE_DIR + type + path).to_s
      else
        @status = "403"
        "Error 403: Invalid path: #{path}"
      end
    end
  end 

  class SaveposXXX
    def post(id, top, left)
      [top, left].each { |pos| pos.gsub!(/px$/, '') }
      $stderr.puts("Got: id: #{id} -> top: #{top}, left: #{left}")
      Positions[id] = [top, left]
      $stderr.puts Positions.inspect if $DEBUG
      ""
    end
  end

  class Upload
    def post
      orig_ext = File.extname(@input.file[:filename]).downcase
      new_file = UPLOAD_DIR + Time.now.strftime("%Y%m%d%H%M%S#{orig_ext}")
      new_file.open("w") do |f|
        f.write(@input.file[:tempfile].read)
      end
      redirect Index
    end
  end

end

module Plemp::Views

  def layout
    xhtml_strict do
      head do
        title "Plemp!"
        link :rel => "stylesheet", :type => "text/css",
             :media => "screen", :href => R(StaticXX, "public", "plemp.css")
        ['prototype', 'scriptaculous', 'dragreg'].each do |js|
          script :src => R(StaticXX, "public", "#{js}.js"), 
                 :type => "text/javascript"
          script :type => "text/javascript" do
            "BaseUrl = \"#{URL()}\";" 
          end 
        end
      end
      body do
        self << yield
      end
    end
  end

  def main
    h1 "Plemp!"
    form :action => R(Upload), :method => "post",
         :enctype => "multipart/form-data" do
      p do 
        input :name => "file", :id => "file", :type => "file"
        input :type => "submit", :value => "Upload!"
      end
    end
    div.draggables! do
      UPLOAD_DIR.entries.select { |f| (UPLOAD_DIR + f).file? }.each do |f|
        next if f.to_s =~ /^\./
        file = UPLOAD_DIR + f
        id = file.basename.to_s
        if Positions.has_key? id
          top, left = Positions[id]
        else
          top, left = ["#{rand(300)}", " #{80 + rand(700)}"]
          Positions[id] = [top, left]
        end
        file_type = `file --brief --mime-type #{file}`.chomp
        mime_type = MIME::Types[file_type].first
        if mime_type.nil?
          mime_type = MIME::Type.new(file_type)
        end
        default_style = "left:#{left}px;top:#{top}px;z-index:0;display:none"
        case mime_type.media_type
          when "image"
            img.draggable :id => id, :style => default_style, 
                          :src => R(StaticXX, "upload", file.basename),
                          :alt => file.basename
          when "video"
            # HTML5 is not supported by Markaby!
            self << \
            "    <video class=\"draggable\" id=\"#{id}\" " +
              "style=\"#{default_style}\" " +
              "src=\"#{R(StaticXX, "upload", file.basename)}\" " + 
              "controls=\"true\">" + "</video>\n"
          when "audio"
            # HTML5 is not supported by Markaby!
            self <<
            "    <audio class=\"draggable\" id=\"#{id}\" " +
              "style=\"#{default_style};height=80px;\" " +
              "src=\"#{R(StaticXX, "upload", file.basename)}\" " + 
              "controls=\"true\">" + "</audio>\n"
          when "text"
            div.draggable :id => id, :style => default_style do
              CodeRay.scan_file(file).div
            end
          else
            span.draggable :id => id, :style => default_style do
              em "#{id}: Unsupported file type!"
            end
        end
        script(:type => "text/javascript") { "setup_draggable('#{id}')" }
      end
    end
  end
end
